// Ejemplo avanzado de paso de opciones desde lnea de parmetros
//
// A los programas a menudo se les quiere pasar desde lnea de
// comandos una serie de opciones, precedidas por un guin. Aunque
// estas opciones se recuperan en argv[] como si fueran un argumento
// ms, su posterior parsing resulta pesado. Este ejmplo muestra cmo
// se usa getopt() para simplificar esta tarea
//
// A este programa se le puede llamar con la opcin -c y entonces
// indica cuntos argumentos ha recibido desde lnea de comandos. Con
// la opcin -t muestra los contenidos (texto) de cada argumento
// recibido. Con la opcin -x slo muestra los argumentos que no son
// opciones. Estas tres opciones se pueden combinar en cualquier
// orden, de modo que sintaxis correctas seran:
//
// parametros2 -x -t algo
// parametros2 -t -x algo
// parametros2 -tx algo
// parametros2 -t algo -x
// parametros2 -tc algo -x
// etc.
//
#include <stdio.h>
#include <unistd.h>

int main(int argc, char *argv[])
{
  int i;
  int opcion;
  // Las tres variables siguientes actuan como "flags" que almacenan
  // el comportamiento deseado del programa. Se asignarn segn las
  // opciones que se encuentren. Por defecto todas estn a cero, por
  // lo que si el programa se llama sin opciones, no hace nada.
  int mostrar_contador=0, mostrar_texto=0, solo_extras=0;

  // Esta es una variable global de getopt()
  // Ponerla a cero indica que no queremos que getopt() genere
  // mensajes de error (programaremos nuestros propios mensajes)
  opterr=0;
  while ((opcion=getopt(argc, argv, "ctx"))!=-1) {
    switch(opcion) {
    case 'c': mostrar_contador=1; break;
    case 't': mostrar_texto=1; break;
    case 'x': solo_extras=1; break;
    case '?': // Opcion desconocida. Su verdadero valor est en optopt
      fprintf(stderr, "Opcin -%c desconocida.\n", optopt);
      fprintf(stderr, "Opciones vlidas:\n");
      fprintf(stderr, "  -c Mostrar numero de argumentos\n");
      fprintf(stderr, "  -t Mostrar texto de los argumentos\n");
      fprintf(stderr, "  -x Solo considerar argumentos no-opcion\n");
      break;
    }
  }

  // Una vez inicializadas las opciones, ejecutamos lo que se nos ha
  // pedido.
  // getopt() ha reordenado los argumentos de la lnea de comandos,
  // poniendo al principio todos los que eran opciones, y dejando para
  // los ltimos los que no lo eran. Ha inicializado la variable
  // optind con el ndice del primer argumento que no es una opcin.
  if (mostrar_contador) {
    if (solo_extras)
      printf("Se han recibido %d parmetros que no son opcin.\n", 
	     argc-optind); 
    else
      printf("Se han recibido en total %d parmetros.\n", argc);
  }
  if (mostrar_texto) {
    if (solo_extras) 
      for (i=optind; i<argc; i++)
	printf("  No-opcin %d: |%s|\n", i-optind, argv[i]);
    else
      for (i=0; i<argc; i++)
	printf("  Parmetro %d: |%s|\n", i, argv[i]);
  }
  return 1;
}
