// Ejemplo de acceso a ficheros usando las funciones de bajo nivel
// (open/read/write) 
//
// La funcion es la misma que la del programa ficheros.c, de hecho,
// los ficheros creados con uno de ellos deberian ser legibles por el
// otro. 
//
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h> // Para open/read/write en unix
#include <string.h>
int main(int argc, char *argv[]) 
{
  int f_texto; // Ahora el tipo fichero es "int" !
  int f_data;
  int x=10;
  int y=47;
  float g=1.0;
  char txt[5]="Hola"; // 5 = 4 letras mas terminador
  char linea[80];     // Para convertir los datos a ASCII

  if (argc<2) { // No hay argumento, modo "crear ficheros"
    // Observar que en Unix, ademas de un modo de apertura se debe
    // indicar los permisos de acceso con que se creara el fichero
    // S_WRITE indica permiso de escritura para el propietario (el
    // creador) 
    f_texto=open("Datos.txt", O_WRONLY|O_CREAT, S_IWRITE);
    if (f_texto==-1) {
      perror("Creando Datos.txt"); exit(-1);
    }
    // Al crear el fichero para datos binarios, bajo Windows/DOS,
    // seria necesario aadir |O_BINARY al modo de apertura. 
    f_data=open("Datos.dat", O_WRONLY|O_CREAT, S_IWRITE);
    if (f_data==-1) {
      perror("Creando Datos.dat"); exit(-1);
    }
    
    // Escribimos los datos como ASCII en Datos.txt
    // sprintf lo convierte en cadena de texto, y write lo escribe en
    // el fichero. Observar el uso de strlen en lugar de sizeof. 
    sprintf(linea, "%s\n", txt); write(f_texto, linea, strlen(linea));
    sprintf(linea, "%d\n", x);   write(f_texto, linea, strlen(linea));
    sprintf(linea, "%d\n", y);   write(f_texto, linea, strlen(linea));
    sprintf(linea, "%f\n", g);   write(f_texto, linea, strlen(linea));
    close(f_texto);
    
    // Ahora escribimos los mismos datos directamente en binario
    write(f_data, txt, 5); // La cadena tiene 4 letras mas terminador
    write(f_data, &x, sizeof(int)); 
    write(f_data, &y, sizeof(int));
    write(f_data, &g, sizeof(float));
    close(f_data);
    
    printf("Ficheros Datos.txt y Datos.dat creados\n");
    return 0;
  }
  else { // Si hay argumento, modo "leer ficheros"
    char buffer[200]; // Para leer el fichero .txt
    // Nota: segun esta implementacion, el fichero no puede tener mas
    // de 200 bytes 
    int n_bytes_leidos;

    f_texto=open("Datos.txt", O_RDONLY);
    if (f_texto==-1) {
      perror("Al abrir Datos.txt");  exit(-1);
    }
    f_data=open("Datos.dat", O_RDONLY);
    if (f_texto==-1) {
      perror("Al abrir Datos.dat"); exit(-1);
    }
    
    // Leemos los datos ASCII de Datos.txt
    // Leemos todo el fichero en un buffer, y extraemos la informacion
    // de ese buffer usando sscanf.
    n_bytes_leidos=read(f_texto, buffer, 200);
    printf("Se han leido %d bytes del fichero Datos.txt\n",
	   n_bytes_leidos);
    close(f_texto);
    // Extraer datos del buffer, usando los tipos apropiados
    sscanf(buffer, "%s\n%d\n%d\n%f\n", txt, &x, &y, &g);

    printf("Datos leidos de Datos.txt\n");
    printf(" txt=%s\n", txt);
    printf(" x=%d\n", x);
    printf(" y=%d\n", y);
    printf(" g=%f\n", g);

    // Leer los datos binarios de Datos.dat es mas sencillo
    read(f_data, txt, 5);
    read(f_data, &x, sizeof(int));
    read(f_data, &y, sizeof(int));
    read(f_data, &g, sizeof(float));
    close(f_data);

    printf("Datos leidos de Datos.dat\n");
    printf(" txt=%s\n", txt);
    printf(" x=%d\n", x);
    printf(" y=%d\n", y);
    printf(" g=%f\n", g);
    return 0;
  }
}
